package pl.edu.pg.eti.kask.rpg.controller.thread;

import lombok.extern.java.Log;
import pl.edu.pg.eti.kask.rpg.controller.SocketController;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Handler for single connection.
 */
@Log
public class SocketConnectionHandler implements Runnable {

    /**
     * Pattern for extracting request URI.
     */
    private static final String REQUEST_URI_PATTERN = "(GET|POST|PUT|DELETE|HEAD)[ ](.+?)[ ]HTTP\\/1.1";

    /**
     * Established connection endpoint.
     */
    private final Socket socket;

    /**
     * Registered controllers.
     */
    private final List<SocketController> controllers;

    /**
     * @param socket      established connection endpoint
     * @param controllers registered controllers
     */
    public SocketConnectionHandler(Socket socket, List<SocketController> controllers) {
        this.socket = socket;
        this.controllers = controllers;
    }

    @Override
    public void run() {
        try (BufferedReader input = new BufferedReader(new InputStreamReader(socket.getInputStream()));
             BufferedWriter output = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream()))) {
            String request = input.readLine();
            List<String> headers = readHeaders(input);
            String uri = extractUri(request);
            SocketController controller = findController(uri);
            if (controller != null) {
                output.write("HTTP/1.1 200 OK");
                output.newLine();
                output.newLine();
                output.write(controller.prepareResponse(uri));
                output.newLine();
            } else {
                output.write("HTTP/1.1 404 Not Found");
                output.newLine();
                output.newLine();
            }
        } catch (IOException ex) {
            log.log(Level.WARNING, ex.getMessage(), ex);
        }

    }

    private SocketController findController(String uri) {
        for (SocketController controller : controllers) {
            if (controller.accepts(uri)) {
                return controller;
            }
        }
        return null;
    }

    private String extractUri(String request) {
        Pattern pattern = Pattern.compile(REQUEST_URI_PATTERN);
        Matcher matcher = pattern.matcher(request);
        if (!matcher.find()) {
            throw new IllegalStateException("Unsupported request pattern: " + request);
        }
        return matcher.group(2);
    }

    private List<String> readHeaders(BufferedReader input) throws IOException {
        List<String> headers = new ArrayList<>();
        String line;
        while (!(line = input.readLine()).equals("")) {
            headers.add(line);
        }
        return headers;
    }

}
