package pl.edu.pg.eti.kask.rpg.repository;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Simple in-memory repository for storing entity classes. It uses {@link Path} do define where data will be stored
 * in binary format.
 *
 * @param <E> type of the entity stored in repository
 */
public abstract class BinaryFileRepository<E extends Serializable> {

    /**
     * Path where data will be stored.
     */
    private final Path path;

    /**
     * @param path path where data will be stored
     */
    public BinaryFileRepository(Path path) {
        this.path = path;
    }

    /**
     * @return all data read from underlying file.
     * @throws IllegalStateException on IO errors or class cast errors
     */
    private Set<E> readAll() throws IllegalStateException {
        if (!Files.exists(path)) {
            return new HashSet<>();
        }
        try (ObjectInputStream ois = new ObjectInputStream(Files.newInputStream(path))) {
            return (Set<E>) ois.readObject();
        } catch (IOException | ClassNotFoundException | ClassCastException ex) {
            throw new IllegalStateException(ex);
        }
    }

    /**
     * Saves all data to underling file.
     *
     * @param data data to be saved
     * @throws IllegalStateException on IO errors
     */
    private void writeAll(Set<E> data) throws IllegalStateException {
        try (ObjectOutputStream oos = new ObjectOutputStream(Files.newOutputStream(path))) {
            oos.writeObject(data);
        } catch (IOException ex) {
            throw new IllegalStateException(ex);
        }
    }

    /**
     * Reads collection from file and returns it.
     *
     * @return list containing all entries
     */
    public List<E> findAll() {
        return new ArrayList<>(readAll());
    }

    /**
     * Remove single element.
     *
     * @param entity element to be removed
     */
    public void delete(E entity) {
        Set<E> set = readAll();
        set.remove(entity);
        writeAll(set);
    }

    /**
     * Removes single element.
     *
     * @param entity element to be removed
     */
    public void add(E entity) {
        Set<E> set = readAll();
        set.add(entity);
        writeAll(set);
    }

}
